!	Grid and interpolate a set of data triples.
!       Remove mean of data values.  Find and set o1,d1,o2,d2.
!	
!%
use sep
use levint2_mod

implicit none
integer 				:: n2, id, nd, niter, warmup
integer, dimension (2)			:: m, a, lag
integer, dimension (:),   allocatable	:: hx, hy
real,    dimension (:),   pointer	:: x, y, z
real,    dimension (:,:), pointer	:: coord, mm, aa, rr
real,    dimension (2)			:: o, d, dx, min, max
real					:: vaver, vrange, vmin, vmax

call sep_init ()

! read data
call from_history (nd, n2)
if (n2 /= 3) call erexit ('bad n2')
allocate (coord (nd,2))
call sep_read (coord)
x => coord (:,1)
y => coord (:,2)

allocate (z (nd))
z = x ; x = y ; y = z		! transpose input
call sep_read (z)

! create model
call from_par ("m1", m (1), 200)
call from_par ("m2", m (2), 200)
call to_history ("n1", m (1))
call to_history ("n2", m (2))
call to_history ("n3", 1)

min (1) = minval (x) ; max (1) = maxval (x)
min (2) = minval (y) ; max (2) = maxval (y)

call from_par ("o1", o (1), min (1))
call from_par ("o2", o (2), min (2))

dx = (max - min) / (m - 1)

call from_par ("d1", d (1), dx (1))
call from_par ("d2", d (2), dx (2))

call to_history ("xmax", max (1))
call to_history ("ymax", max (2))
call to_history ("label1", "longitude" )
call to_history ("label2", "latitude")

vmin = minval (z)
vmax = maxval (z)

vaver  = (vmax + vmin) * 0.5
vrange = (vmax - vmin) * 0.5
z = (vaver - z)/ vrange

! create filter
call from_par ("a1", a (1), 5)
call from_par ("a2", a (2), 3)
allocate (rr (m (1), m (2) + a (2)))
mm => rr (:, : m (2))
mm = 0.

allocate (hx (nd), hy (nd))
hx = 1.5 + (x - o (1))/d (1)
hy = 1.5 + (y - o (2))/d (2)
do id = 1, nd
	if (0 < hx (id) .and. hx (id) < m (1) .and. 	&
	    0 < hy (id) .and. hy (id) < m (2)) 		&
		mm (hx (id), hy (id)) = z (id)
end do
deallocate (hx, hy)
call to_history ("n1",m (1), "last")
call to_history ("n2",m (2), "last")
call to_history ("n3",1    , "last")
call to_history ("esize", 4, "last")
call sep_write (mm, "last")
mm = 0.

aa => rr (: a (1), 1 + m (2) :)
aa = 0. 
lag (1) = 1 + a (1)/2
lag (2) = 1 

aa (lag (1) - 1, 2) = 1.
aa (lag (1), 1 : 3) = (/ 1., -4., 1. /)
aa (lag (1) + 1, 2) = 1.

call from_par ("niter" , niter , 20)
call from_par ("warmup", warmup, 20)
call sep_close()	

call levint2 (niter, warmup, coord, z, nd, o, d, aa, mm, &
					rr, size (rr), lag)

call sep_write (mm)

deallocate (rr, coord, z)

call exit (0)
end


