module levint2_mod
  use tcai2
  use peftcreg2
  use int2
  use interp
  use setfree
  use cgstep_mod
  use solver_mod

  implicit none

contains

  subroutine levint2 (niter, warmup, xy, dd, nd, o, d, aa, mm, rr, nr, lag)
    integer,               intent (in)    :: niter, warmup, nd, nr
    real, dimension (2),   intent (in)    :: o, d
    real, dimension (:,:), pointer        :: aa, mm, xy
    real, dimension (:),   intent (in)    :: dd
    real, dimension (nr),  intent (inout) :: rr
    integer, dimension (2), intent (in)   :: lag

    integer, dimension (2)                :: nm, na, nx
    integer                               :: nreg
    logical, dimension (:,:), pointer     :: ma

    nm (1) = size (mm, 1) ; na (1) = size (aa, 1) ; nx (1) = nm (1) 
    nm (2) = size (mm, 2) ; na (2) = size (aa, 2) ; nx (2) = nm (2) + na (2)

    nreg = product (nm + na)

    allocate (ma (na (1), na (2)))
    call set_free (ma, lag)

    call int2_init (xy, o, d, nm, bilin_int, (/ 2, 2 /))
    call peftcreg2_init (ma, aa, mm)

    call cgstep_init ()
    call solver (lint2_op, cgstep, &
    reg =     tcai2_op, nreg = nreg, &
    niter = min (niter, warmup), &
    x0 = rr, x = rr, dat = dd)
    call solver (lint2_op, cgstep, &
    reg = peftcreg2_op, nreg = nreg, nlreg = tcai2_op, &
    niter = max (0, niter - warmup), &
    x0 = rr, x = rr, dat = dd)
    call cgstep_close ()

    call int2_close ()

    deallocate (ma)
  end subroutine levint2

end module levint2_mod

