module misif_mod
  use peftc
  use tcai1
  use cgstep_mod
  use solver_mod

  ! MISSIF -- find MISSing Input data and Filter 
  ! on 1-axis by min power out.

  implicit none

contains

  ! fill in missing data on 1-axis by 
  ! minimizing power out of a given filter.
  !
  ! xx - in: known data, out: interpolated data
  ! aa - prediction - error filter
  ! mx - mask

  subroutine misif (niter, xx, aa, mm)
    integer, intent (in)                               :: niter
    real,    dimension (:)                             :: xx, aa 
    logical, dimension (:)                             :: mm

    real,    dimension (:), pointer                    :: xk, bb
    real,    dimension (size (xx) + size (aa))         :: dat
    real,    dimension (size (xx) + size (aa)), target :: x
    integer                                            :: nx

    nx = size (xx)

    x = 0.
    xk => x (1 : nx  ) ; where (mm (1 : nx )) xk = xx
    bb => x (1 + nx :) ; where (mm (1 + nx:)) bb = 1.

    call peftc_init (bb, xk)

    dat = 0.
    call cgstep_init ()
    call solver (peftc_op, cgstep, niter = niter, x0 = x, x = x, &
    dat = dat, nloper = tcai1_op, mask = mm)
    call cgstep_close ()

    xx = xk ; aa = bb
  end subroutine misif

end module misif_mod

